"""Initialize the manager."""
import subprocess
import os
from pathlib import Path


class Manager():
    """Manager class."""

    def __init__(
            self, suite=None, mode=None, dev=False, beta=False, version=None):
        """Initialize the class."""
        self.beta = beta
        self.dev = dev
        self.exist = False
        self.mode = mode
        self.suite = suite
        self.suites = None
        self.version = version

    def is_installed(self):
        """Verify that hassbian-config is installed."""
        self.exist = bool(Path("/usr/local/bin/hassbian-config").is_file())
        return self.exist

    def get_suites(self):
        """Get a list of available suites."""
        self.suites = []
        for file in os.listdir("/opt/hassbian/suites"):
            if file.endswith(".sh"):
                self.suites.append(file[:-3])
        return sorted(self.suites)

    def get_version(self):
        """Return the version of hassbian-config."""
        command = subprocess.Popen(
            ['hassbian-config', '--version'], stdout=subprocess.PIPE)
        out = command.communicate()
        version = str(out[0][:-1]).split("'")[1]
        return version

    def manage_suite(self):
        """Upgrade a suite with hassbian-config."""
        self.get_suites()
        if self.suite in self.suites:
            cmd = ['sudo', 'hassbian-config', self.mode]
            if self.dev:
                cmd.append(self.suite)
                cmd.append("--dev")
            elif self.beta:
                cmd.append(self.suite)
                cmd.append("--beta")
            else:
                if self.version:
                    cmd.append("{}={}".format(self.suite, self.version))
                else:
                    cmd.append(self.suite)
            cmd.append("--accept")
            cmd.append("--force")
            subprocess.call(cmd)

    def os_upgrade(self):
        """Upgrade the base OS."""
        subprocess.call(
            ['sudo', 'hassbian-config', 'upgrade', 'hassbian',
             '--accept', '--dev'])

    def log(self):
        """Return log output."""
        logentry = ''
        filename = "/tmp/hassbian.log"
        if os.path.isfile(filename):
            with open(filename, 'r') as myfile:
                logentry = myfile.read()
        return logentry

    def suite_installed(self):
        """Return a bool that indicates if the suite is installed."""
        suite_state = False
        filename = "/srv/homeassistant/hassbian/control/{}".format(self.suite)
        if os.path.isfile(filename):
            with open(filename, 'r') as myfile:
                state = myfile.read().split('\n')[0].split('SCRIPTSTATE=')[1]
                if state == 'installed':
                    suite_state = True
        return suite_state



"""Enable CLI."""
import click


@click.command()
@click.option('--port', '-p', default=9999, help='Port number.')
@click.option('--username', '-U', default='pi', help='Username.')
@click.option('--password', '-P', default='raspberry', help='Password.')
@click.option('--nu_auth', is_flag=True, help='Disable auth.')
def cli(port, username, password, no_auth):
    """CLI for this package."""
    from pyhassbian.server import run_server
    run_server(port, username, password, no_auth)


cli()  # pylint: disable=E1120



"""Static texts."""
STYLE = """
<head>
    <title>Hassbian Manager</title>
    <link rel="shortcut icon" href="/static/package.ico" />
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/css/font-awesome.min.css">
    <link href='http://fonts.googleapis.com/css?family=Roboto' rel='stylesheet' type='text/css'>
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/materialize/1.0.0/css/materialize.min.css">
    <link rel="stylesheet" href="static/style.css">
    <script src="https://cdnjs.cloudflare.com/ajax/libs/materialize/1.0.0/js/materialize.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/jquery/3.1.1/jquery.min.js"></script>
    <script src="static/action.js"></script>
</head>
"""

HEADER = """
<header>
  <nav>
    <div class="nav-wrapper">
      <a href="/" class="brand-logo">&nbsp;&nbsp;&nbsp;Hassbian Manager</a>
      <ul id="nav-mobile" class="right hide-on-med-and-down">
        <li><a href="/">Suites</a></li>
        <li><a href="/log">Log</a></li>
      </ul>
    </div>
  </nav>
</header>
"""

CARD = """
  <div class="row">
    <div class="col s12">
      <div class="card blue-grey darken-1">
        <div class="card-content white-text" id="{title}">
          <span class="card-title">{title}</span>
          <p>{content}</p>
        </div>
        <div class="card-action">
          <a href="{more}">More info</a>
        </div>
      </div>
    </div>
  </div>
"""

SUITE = """
  <div class="row suiterow">
    <div class="col s12">
      <div class="card blue-grey darken-1">
        <div class="card-content white-text" id="{title}">
          <span class="card-title">{title}</span>
          <p>{content}</p>
        </div>
        <div class="card-action">
          {buttons}
        </div>
      </div>
    </div>
  </div>
"""

LOG = """
  <div class="row">
    <div class="col s12">
      <div class="card blue-grey darken-1">
        <div class="card-content white-text">
          <p>{}</p>
        </div>
        <div class="card-action">
          <a href="/log">Refresh</a>
        </div>
      </div>
    </div>
  </div>
</main>
"""



"""Webserver."""
import os
from aiohttp import web
from pyhassbian.manager import Manager
from pyhassbian.generated import generated
from aiohttp_basicauth import BasicAuthMiddleware

DIRPATH = os.path.dirname(os.path.realpath(__file__))

SUITES = ['appdaemon',
          'cloud9',
          'custom-component-store',
          'fail2ban',
          'hassbian-script',
          'hassbian',
          'homeassistant',
          'homebridge',
          'hue',
          'libcec',
          'manager',
          'mosquitto',
          'razberry',
          'samba',
          'tradfri',
          'webterminal',
          'zigbee2mqtt']


async def html(request):
    """Serve a HTML site."""
    content = generated.STYLE
    content += generated.HEADER

    content += '<main class="suites">'

    suites = await get_data()

    for suite in suites:
        if suite in SUITES:
            installed = Manager(suite=suite).suite_installed()
            if installed or suite in ['homeassistant', 'hassbian-script', 'hassbian']:
                title = "{} (installed)".format(suite)
            else:
                title = suite
            filename = "/opt/hassbian/suites/{}.sh".format(suite)
            with open(filename, 'r') as myfile:
                myfile = myfile.read().replace('\n', '')
                myfile = myfile.replace('\\n', '').replace('printf ', '')
                myfile = myfile.replace('\\', '')
                shortdesc = myfile.split('show-short-info {')[1].split('}')[0]
                shortdesc = shortdesc.replace('echo "', '').replace('"', '')
            content += generated.CARD.format(
                title=title, content=shortdesc, more=suite)

    content += '</main>'
    return web.Response(body=content, content_type="text/html")


async def suiteview(request):
    """Serve a HTML site."""
    suite = request.match_info['suite']
    has_upgrade = False
    has_install = False
    has_remove = False

    buttons = ''

    docs = "https://github.com/home-assistant/hassbian-scripts/blob/master/"
    if suite == 'hassbian-script':
        docs += "docs/hassbian_config.md"
    else:
        docs += "docs/{}.md".format(suite)

    content = generated.STYLE
    content += generated.HEADER

    content += '<main class="suite">'

    filename = "/opt/hassbian/suites/{}.sh".format(suite)
    with open(filename, 'r') as myfile:
        myfile = myfile.read().replace('\n', '</br>')
        myfile = myfile.replace('\\n', '').replace('printf ', '')
        myfile = myfile.replace('\\', '')

        if '-install-package' in myfile:
            has_install = True
        if '-upgrade-package' in myfile:
            has_upgrade = True
        if '-remove-package' in myfile:
            has_remove = True

        shortdesc = myfile.split('show-short-info {')[1].split('}')[0]
        shortdesc = shortdesc.replace('echo "', '').replace('"', '')

        longdesc = myfile.split('show-long-info {')[1].split('}')[0]
        longdesc = longdesc.replace('echo "', '').replace('"', '')

        if longdesc == shortdesc:
            longdesc = ''

    installed = Manager(suite=suite).suite_installed()

    if installed or suite in ['homeassistant', 'hassbian-script', 'hassbian']:
        title = "{} (installed)".format(suite)
    else:
        title = suite

    body = shortdesc
    body += '</br>'
    body += longdesc

    if has_install:
        if suite != 'homebridge':
            buttons += '<a href="/{}/install" class="install">Install</a>'.format(suite)
    if installed or suite in ['homeassistant', 'hassbian-script', 'hassbian']:
        if has_upgrade:
            buttons += '<a href="/{}/upgrade" class="upgrade">Upgrade</a>'.format(suite)
        if has_remove:
            buttons += '<a href="/{}/remove" class="remove">Remove</a>'.format(suite)

    buttons += '<a href="{}" target="_blank">Documentation</a>'.format(docs)

    content += generated.SUITE.format(
        title=title, content=body, buttons=buttons)
    content += '<div class="loader"></div>'
    content += '</main>'
    return web.Response(body=content, content_type="text/html")


async def log(request):
    """Serve a HTML site."""
    content = generated.STYLE
    content += generated.HEADER

    content += '<main class="log">'

    lastlog = await get_log()
    lastlog = lastlog.replace('\n', '</br>')

    content += generated.LOG.format(lastlog)

    content += '</main>'
    return web.Response(body=content, content_type="text/html")


async def json(request):
    """Serve the response as JSON."""
    json_data = await get_data()
    return web.json_response(json_data)


async def install(request):
    """Install suite"""
    suite = request.match_info['suite']
    Manager(suite=suite, mode='install').manage_suite()
    raise web.HTTPFound('/' + suite)


async def upgrade(request):
    """upgrade suite"""
    suite = request.match_info['suite']
    if suite == 'hassbian':
        Manager().os_upgrade()
    else:
        Manager(suite=suite, mode='upgrade').manage_suite()
    raise web.HTTPFound('/' + suite)


async def remove(request):
    """remove suite"""
    suite = request.match_info['suite']
    Manager(suite=suite, mode='remove').manage_suite()
    raise web.HTTPFound('/' + suite)


async def get_data():
    """Get version data."""
    return Manager().get_suites()


async def get_log():
    """Get version data."""
    return Manager().log()


def run_server(port, username, password, no_auth):
    """Run the server."""
    if not no_auth:
        auth = BasicAuthMiddleware(
            username=username, password=password, force=True)
        app = web.Application(middlewares=[auth])
    else:
        app = web.Application()
    app.router.add_route('GET', r'/', html, name='html')
    app.router.add_route('GET', r'/log', log, name='log')
    app.router.add_route('GET', r'/json', json, name='json')
    app.router.add_route('GET', r'/{suite}', suiteview, name='suiteview')
    app.router.add_route('GET', r'/{suite}/install', install, name='install')
    app.router.add_route('GET', r'/{suite}/upgrade', upgrade, name='upgrade')
    app.router.add_route('GET', r'/{suite}/remove', remove, name='remove')
    app.router.add_static('/static/', path=str(DIRPATH + '/static/'))
    web.run_app(app, port=port)
